#   include	<stdio.h>
#   include	<stdlib.h>
#   include	<string.h>
#   include	<iconv.h>
#   include	<ind.h>
#   include	<charnames.h>
#   include	<uniUtf8.h>
#   include	<appDebugon.h>
#   include	"affix.h"

void affixInitThrough(	AffixThrough *	at )
    {
    at->atEncodingIn= (const char *)0;
    at->atEncodingOut= (const char *)0;
    at->atIconv= (iconv_t)-1;
    at->atVoidInd= (void *)0;
    at->atRules= (void *)0;
    at->atSuffixInd= (void *)0;

    utilInitIndexMapping( &(at->atSuffixMapping) );

    return;
    }

void affixCleanThrough(	AffixThrough *	at )
    {
    if  ( at->atIconv != (iconv_t)-1 )
	{ iconv_close( at->atIconv );	}

    if  ( at->atVoidInd )
	{ indFree( at->atVoidInd );	}
    if  ( at->atSuffixInd )
	{ indFree( at->atSuffixInd );	}

    if  ( at->atRules )
	{ affixFreeRules( at->atRules );	}

    utilCleanIndexMapping( &(at->atSuffixMapping) );

    return;
    }

int affixToUtf16(	iconv_t *		icv,
			unsigned short *	scratch,
			int			scratchSize,
			const char *		word,
			int			wordLength )
    {
    size_t		ileft= wordLength;
    size_t		oleft= scratchSize* sizeof( unsigned short );
    char *		ibuf= (char *)word;
    char *		obuf= (char *)scratch;

    char *		iibuf= ibuf;

    while( oleft > 0						&&
	   ileft > 0						&&
	   iconv( icv, &ibuf, &ileft, &obuf, &oleft ) >= 0	)
	{
	if  ( ibuf == iibuf )
	    { /*XXDEB(ibuf,iibuf);*/ break;	}

	iibuf= ibuf;
	}

    if  ( ileft > 0 )
	{ LDEB(ileft);	}

    *(unsigned short *)obuf= 0;
    return 0;
    }

void affixLogUtf16( const unsigned short * w )
    {
    while( *w )
	{
	unsigned char	scratch[10];
	int		step;
	int		j;

	step= uniPutUtf8( scratch, *w );
	if  ( step < 1 )
	    { LLDEB(*w,step);	}

	for ( j= 0; j < step; j++ )
	    { appDebug( "%c", scratch[j] );	}

	w++;
	}

    appDebug( "\n" );

    return;
    }

/************************************************************************/
/*									*/
/*  Book keeping.							*/
/*									*/
/************************************************************************/

void affixInitRule(	AffixRule *	ar )
    {
    int		i;

    ar->arWhatSide= 0;
    ar->arShift= 0;
    ar->arCross= 0;
    ar->arMaskLength= 0;
    ar->arNext= (AffixRule *)0;
    
    for ( i= 0; i < AFFMAXLENGTH; i++ )
	{
	ar->arMasks[i].amNegative= 0;
	utilInitIndexSet( &(ar->arMasks[i].amSet) );
	}

    ar->arReplaceFrom.rsLength= 0;
    ar->arReplaceFrom.rsUtf16[0]= 0;
    ar->arReplaceWith.rsLength= 0;
    ar->arReplaceWith.rsUtf16[0]= 0;
    ar->arSuffixNumber= -1;

    return;
    }

void affixFreeRule(	AffixRule *	ar )
    {
    int		i;

    for ( i= 0; i < AFFMAXLENGTH; i++ )
	{
	utilCleanIndexSet( &(ar->arMasks[i].amSet) );
	}

    return;
    }

/************************************************************************/
/*									*/
/*  Bookkeeping								*/
/*									*/
/************************************************************************/

void affixInitRules(	AffixRules *	ars )
    {
    int		i;

    ars->arStringChars= (StringChar *)0;
    ars->arStringCharCount= 0;

    utilInitIndexSet( &(ars->arWordChars) );
    utilInitIndexSet( &(ars->arBoundaryChars) );

    for ( i= 0; i < 256; i++ )
	{ ars->arsRules[i]= (AffixRule *)0;	}

    return;
    }

void affixFreeRules(	AffixRules *	ars )
    {
    int		i;

    for ( i= 0; i < 256; i++ )
	{
	AffixRule *	r= ars->arsRules[i];

	while( r )
	    {
	    AffixRule *	n= r->arNext;

	    affixFreeRule( r );
	    r= n;
	    }
	}

    if  ( ars->arStringChars )
	{ free( ars->arStringChars );	}

    free( ars );

    return;
    }

